	 ;1-wire reset

w1_reset:
	ldi	a1, 480 * xtal / 1000000 / 7	;	max 3.7MHz !
_w1r1:	cbi	w1_out, w1_pin			;2
	sbi	w1_ddr, w1_pin				;2	low
	subi	a1, 1					;1	C = 0
	brne	_w1r1					;2

	ldi	a1, 480 * xtal / 1000000 / 7
	cli
_w1r2:	cbi	w1_ddr, w1_pin			;2	high
	sbis	w1_in, w1_pin			;1	presence detect
	cpi	a1, 465 * xtal / 1000000 / 7;1
	dec	a1				;1
	brne	_w1r2					;2
	sbis	w1_in, w1_pin			;	check short circuit
	clc
	reti					;	int enable


	 ;Read / Write Bit on 1-wire

;Input	C = bit
;Output	C = bit
;Used	a1
;
w1_bit_rd:
	sec							;set carry
w1_bit_wr:
w1_bit_io:
	cli							;global interrupt disable
	sbi	w1_ddr, w1_pin			;low
	ldi	a1, 15 * xtal / 1000000 / 3
	brcc	_w1io1
	cbi	w1_ddr, w1_pin			;write 1 bit
_w1io1:	dec	a1
	brne	_w1io1
	sbis	w1_in, w1_pin		;sample input
	clc
	ldi	a1, 45 * xtal / 1000000 / 3
_w1io2:	dec	a1
	brne	_w1io2
	cbi	w1_ddr, w1_pin			;high
	reti						;int enable


	 ;Read / Write byte on 1-wire

;Input	a0 = byte
;Output a0 = byte
;Used	a0, a1, a2
;
w1_byte_rd:
	ldi	a0, 0xFF
w1_byte_wr:
	ldi	a2, 8
_w1wr1:
	ror	a0						;lsb first
	cli							;because 3th stack level
	rcall	w1_bit_io
	dec	a2
	brne	_w1wr1
	ror	a0						;last bit in
	ret

	 ;Read Temperature Sensors
measure:
	ldi		a0, 0x7F				;mark no valid result
	mov		temp1h, a0
	mov		temp2h, a0
	mov 	temp3h, a0
	ldi		a0, search_first		;search sensor
_rte1:	rcall	w1_rom_search
	cpi		a0, presence_err
	breq	_rte5
	mov		a3, a0
	ldi		zl, romcode_ptr
	ld		a1, z					;device code:
	cpi		a1, 0x10
	brne	_rte4				;no DS18s20
	ldi		a0, READ_T				;read temperature
	rcall	w1_byte_wr
	rcall	w1_byte_rd			;low byte
	mov		temp3l, temp2l
	mov		temp2l, temp1l
	mov		temp1l, a0
	rcall	w1_byte_rd			;high byte
	mov		temp3h, temp2h
	mov		temp2h, temp1h
	mov		temp1h, a0
_rte4:	mov	a0, a3
	cpi		a0, LAST_DEVICE
	brne	_rte1
_rte5:

	 ;Start Next Measurement
	rcall	w1_reset
	cpi	a0, presence_err
	breq	_rte6
	ldi	a0, SKIP_ROM
	rcall	w1_byte_wr
	ldi	a0, CONVERT_T
	rcall	w1_byte_wr			;start measuring
	sbi	w1_out, w1_pin
	sbi	w1_ddr, w1_pin			;parasite power on
_rte6:	ret

	 ;ROM Search

;Input	a0 = bit to resolve
;Output	a0 = next bit to resolve
;Used	a0, a1, a2, a3, zl
w1_rom_search:
	mov	a3, a0
	rcall	w1_reset
	ldi	a0, presence_err
	brcc	_wrs7
	ldi	a0, search_rom
	rcall	w1_byte_wr
	ldi	zl, romcode_ptr		; point to rom code memory
	ldi	a0, last_device
	ldi	a2, 8 * 16 - 1		; 8 bytes a 8 bit
_wrs1:	rcall	w1_bit_rd		; read bit
	sbc	a1, a1			; H = C
	rcall	w1_bit_rd		; read complement bit
	ld	a1, z
	brhc	_wrs2
	brcc	_wrs4			; bit = 1
	ldi	a0, data_err
	ret
_wrs2:	brcs	_wrs5			; bit = 0
	cp	a2, a3
	breq	_wrs5			; bit = 0
	brcs	_wrs3			; bit = 1, resolve
	sbrs	a1, 0
	rjmp	_wrs5			; bit = 0
_wrs3:	mov	a0, a2			; bit = 1
_wrs4:	sec
	rjmp	_wrs6
_wrs5:	clc
_wrs6:	ror	a1
	st	z, a1
	rol	a1
	rcall	w1_bit_wr
	subi	a2, 2			; after 8 times: H = 1
	brhc	_wrs1
	inc	zl			; next byte
	brcc	_wrs1
_wrs7:	ret
